package prt2.age.gui;

import android.animation.AnimatorSet;
import android.animation.ObjectAnimator;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.RectF;
import android.view.View;
import android.view.animation.BounceInterpolator;
import prt2.age.graph.Arc;
import prt2.age.graph.LocatedGraph;
import prt2.age.graph.LocatedNode;
import prt2.age.graph.Point;

/**
 *
 * @author trust56
 */
public class AndroidGraph<T> extends LocatedGraph<T> {

    private View view;

    void draw(Canvas g) {
        Paint p = new Paint();
        p.setAntiAlias(true);
        for (Arc<T, LocatedNode<T>> arc : arcs) {
            if (selectedArc == arc) {
                p.setColor(Color.GREEN);
            } else {
                p.setColor(Color.BLACK);
            }
            int x1 = arc.edge1.x;
            int y1 = arc.edge1.y;
            if (arc.edge1 != arc.edge2) {
                int x2 = arc.edge2.x;
                int y2 = arc.edge2.y;
                g.drawLine(x1, y1, x2, y2, p);
            } else {
                g.drawOval(new RectF(x1, y1, x1 + NODE_RADIUS * 2, y1 + NODE_RADIUS * 2), p);
            }
        }
        for (LocatedNode<T> node : nodes) {
            int x = node.x;
            int y = node.y;
            p.setColor(Color.BLACK);
            g.drawOval(new RectF(x - NODE_RADIUS, y - NODE_RADIUS,
                    x + NODE_RADIUS, y + NODE_RADIUS), p);
            if (selectedNode == node) {
                p.setColor(Color.GREEN);
            } else {
                p.setColor(Color.WHITE);
            }
            g.drawOval(new RectF(x - NODE_RADIUS + 1, y - NODE_RADIUS + 1,
                    x + NODE_RADIUS - 1, y + NODE_RADIUS - 1), p);
            p.setColor(Color.BLACK);
            g.drawText(node.element.toString(), x - 2, y + 4, p);
        }
    }

    @Override
    public void graphChanged() {
        if (view != null) {
            view.postInvalidate();
        }
    }

    public void setView(View view) {
        this.view = view;
    }
    private final int MOVE_ANIM_TRESHOLD = 10;

    @Override
    public void dragPoint(Point point) {
        if (selectedNode == null) {
            return;
        }
        if (distance(point, selectedNode) > MOVE_ANIM_TRESHOLD) {
            ObjectAnimator xAnim = ObjectAnimator.ofInt(new NodeLocationAnimatorWrapper(selectedNode), "x", selectedNode.x, point.x).setDuration(1000);
            ObjectAnimator yAnim = ObjectAnimator.ofInt(new NodeLocationAnimatorWrapper(selectedNode), "y", selectedNode.y, point.y).setDuration(1000);
            AnimatorSet animatorSet = new AnimatorSet();
            animatorSet.play(xAnim).with(yAnim);
            //animatorSet.setInterpolator(new BounceInterpolator());
            animatorSet.start();
        } else {
            super.dragPoint(point);
        }
    }

    private class NodeLocationAnimatorWrapper {

        private LocatedNode<T> node;

        public NodeLocationAnimatorWrapper(LocatedNode<T> node) {
            this.node = node;
        }

        public void setX(int x) {
            node.x = x;
//            graphChanged();
        }
        public void setY(int y) {
            node.y = y;
            graphChanged();
        }
    }

}
